# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import RelaxaiError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)

if TYPE_CHECKING:
    from .resources import chat, tools, models, embeddings, deep_research
    from .resources.chat import ChatResource, AsyncChatResource
    from .resources.tools import ToolsResource, AsyncToolsResource
    from .resources.models import ModelsResource, AsyncModelsResource
    from .resources.embeddings import EmbeddingsResource, AsyncEmbeddingsResource
    from .resources.deep_research import DeepResearchResource, AsyncDeepResearchResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Relaxai", "AsyncRelaxai", "Client", "AsyncClient"]


class Relaxai(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Relaxai client instance.

        This automatically infers the `api_key` argument from the `RELAXAI_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RELAXAI_API_KEY")
        if api_key is None:
            raise RelaxaiError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RELAXAI_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("RELAXAI_BASE_URL")
        if base_url is None:
            base_url = f"https://api.relax.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        from .resources.embeddings import EmbeddingsResource

        return EmbeddingsResource(self)

    @cached_property
    def models(self) -> ModelsResource:
        from .resources.models import ModelsResource

        return ModelsResource(self)

    @cached_property
    def tools(self) -> ToolsResource:
        from .resources.tools import ToolsResource

        return ToolsResource(self)

    @cached_property
    def deep_research(self) -> DeepResearchResource:
        from .resources.deep_research import DeepResearchResource

        return DeepResearchResource(self)

    @cached_property
    def with_raw_response(self) -> RelaxaiWithRawResponse:
        return RelaxaiWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RelaxaiWithStreamedResponse:
        return RelaxaiWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def health(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Check the health of the service."""
        return self.get(
            "/v1/health",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncRelaxai(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncRelaxai client instance.

        This automatically infers the `api_key` argument from the `RELAXAI_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RELAXAI_API_KEY")
        if api_key is None:
            raise RelaxaiError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RELAXAI_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("RELAXAI_BASE_URL")
        if base_url is None:
            base_url = f"https://api.relax.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        from .resources.embeddings import AsyncEmbeddingsResource

        return AsyncEmbeddingsResource(self)

    @cached_property
    def models(self) -> AsyncModelsResource:
        from .resources.models import AsyncModelsResource

        return AsyncModelsResource(self)

    @cached_property
    def tools(self) -> AsyncToolsResource:
        from .resources.tools import AsyncToolsResource

        return AsyncToolsResource(self)

    @cached_property
    def deep_research(self) -> AsyncDeepResearchResource:
        from .resources.deep_research import AsyncDeepResearchResource

        return AsyncDeepResearchResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncRelaxaiWithRawResponse:
        return AsyncRelaxaiWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRelaxaiWithStreamedResponse:
        return AsyncRelaxaiWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    async def health(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Check the health of the service."""
        return await self.get(
            "/v1/health",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class RelaxaiWithRawResponse:
    _client: Relaxai

    def __init__(self, client: Relaxai) -> None:
        self._client = client

        self.health = to_raw_response_wrapper(
            client.health,
        )

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithRawResponse:
        from .resources.embeddings import EmbeddingsResourceWithRawResponse

        return EmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def models(self) -> models.ModelsResourceWithRawResponse:
        from .resources.models import ModelsResourceWithRawResponse

        return ModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithRawResponse:
        from .resources.tools import ToolsResourceWithRawResponse

        return ToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def deep_research(self) -> deep_research.DeepResearchResourceWithRawResponse:
        from .resources.deep_research import DeepResearchResourceWithRawResponse

        return DeepResearchResourceWithRawResponse(self._client.deep_research)


class AsyncRelaxaiWithRawResponse:
    _client: AsyncRelaxai

    def __init__(self, client: AsyncRelaxai) -> None:
        self._client = client

        self.health = async_to_raw_response_wrapper(
            client.health,
        )

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithRawResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithRawResponse

        return AsyncEmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithRawResponse:
        from .resources.models import AsyncModelsResourceWithRawResponse

        return AsyncModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithRawResponse:
        from .resources.tools import AsyncToolsResourceWithRawResponse

        return AsyncToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def deep_research(self) -> deep_research.AsyncDeepResearchResourceWithRawResponse:
        from .resources.deep_research import AsyncDeepResearchResourceWithRawResponse

        return AsyncDeepResearchResourceWithRawResponse(self._client.deep_research)


class RelaxaiWithStreamedResponse:
    _client: Relaxai

    def __init__(self, client: Relaxai) -> None:
        self._client = client

        self.health = to_streamed_response_wrapper(
            client.health,
        )

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import EmbeddingsResourceWithStreamingResponse

        return EmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def models(self) -> models.ModelsResourceWithStreamingResponse:
        from .resources.models import ModelsResourceWithStreamingResponse

        return ModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithStreamingResponse:
        from .resources.tools import ToolsResourceWithStreamingResponse

        return ToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def deep_research(self) -> deep_research.DeepResearchResourceWithStreamingResponse:
        from .resources.deep_research import DeepResearchResourceWithStreamingResponse

        return DeepResearchResourceWithStreamingResponse(self._client.deep_research)


class AsyncRelaxaiWithStreamedResponse:
    _client: AsyncRelaxai

    def __init__(self, client: AsyncRelaxai) -> None:
        self._client = client

        self.health = async_to_streamed_response_wrapper(
            client.health,
        )

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithStreamingResponse

        return AsyncEmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithStreamingResponse:
        from .resources.models import AsyncModelsResourceWithStreamingResponse

        return AsyncModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithStreamingResponse:
        from .resources.tools import AsyncToolsResourceWithStreamingResponse

        return AsyncToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def deep_research(self) -> deep_research.AsyncDeepResearchResourceWithStreamingResponse:
        from .resources.deep_research import AsyncDeepResearchResourceWithStreamingResponse

        return AsyncDeepResearchResourceWithStreamingResponse(self._client.deep_research)


Client = Relaxai

AsyncClient = AsyncRelaxai



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .model import Model as Model
from .shared import (
    OpenAIUsage as OpenAIUsage,
    OpenAIPromptTokensDetails as OpenAIPromptTokensDetails,
    OpenAICompletionTokensDetails as OpenAICompletionTokensDetails,
)
from .model_list import ModelList as ModelList
from .function_call import FunctionCall as FunctionCall
from .tool_response import ToolResponse as ToolResponse
from .health_response import HealthResponse as HealthResponse
from .embedding_response import EmbeddingResponse as EmbeddingResponse
from .function_call_param import FunctionCallParam as FunctionCallParam
from .stream_options_param import StreamOptionsParam as StreamOptionsParam
from .deepresearch_response import DeepresearchResponse as DeepresearchResponse
from .content_filter_results import ContentFilterResults as ContentFilterResults
from .chat_completion_message import ChatCompletionMessage as ChatCompletionMessage
from .chat_completion_response import ChatCompletionResponse as ChatCompletionResponse
from .tool_execute_code_params import ToolExecuteCodeParams as ToolExecuteCodeParams
from .function_definition_param import FunctionDefinitionParam as FunctionDefinitionParam
from .deep_research_create_params import DeepResearchCreateParams as DeepResearchCreateParams
from .chat_completion_message_param import ChatCompletionMessageParam as ChatCompletionMessageParam
from .chat_create_completion_params import ChatCreateCompletionParams as ChatCreateCompletionParams
from .embedding_create_embedding_params import EmbeddingCreateEmbeddingParams as EmbeddingCreateEmbeddingParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .deep_research import (
    DeepResearchResource,
    AsyncDeepResearchResource,
    DeepResearchResourceWithRawResponse,
    AsyncDeepResearchResourceWithRawResponse,
    DeepResearchResourceWithStreamingResponse,
    AsyncDeepResearchResourceWithStreamingResponse,
)

__all__ = [
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "DeepResearchResource",
    "AsyncDeepResearchResource",
    "DeepResearchResourceWithRawResponse",
    "AsyncDeepResearchResourceWithRawResponse",
    "DeepResearchResourceWithStreamingResponse",
    "AsyncDeepResearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FunctionDefinitionParam", "Parameters"]


class Parameters(TypedDict, total=False):
    pass


class FunctionDefinitionParam(TypedDict, total=False):
    name: Required[str]

    parameters: Required[Parameters]

    description: str

    strict: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr
from .stream_options_param import StreamOptionsParam
from .function_definition_param import FunctionDefinitionParam
from .chat_completion_message_param import ChatCompletionMessageParam

__all__ = ["DeepResearchCreateParams", "ResponseFormat", "ResponseFormatJsonSchema", "Tool"]


class DeepResearchCreateParams(TypedDict, total=False):
    messages: Required[Iterable[ChatCompletionMessageParam]]

    model: Required[str]

    max_tokens: int

    response_format: ResponseFormat

    stop: SequenceNotStr[str]

    stream: bool

    stream_options: StreamOptionsParam

    temperature: float

    tool_choice: object

    tools: Iterable[Tool]

    top_p: float


class ResponseFormatJsonSchema(TypedDict, total=False):
    description: str

    name: str

    schema: str

    strict: bool


class ResponseFormat(TypedDict, total=False):
    json_schema: ResponseFormatJsonSchema

    type: str


class Tool(TypedDict, total=False):
    type: Required[str]

    function: FunctionDefinitionParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from relaxai import Relaxai, AsyncRelaxai
from tests.utils import assert_matches_type
from relaxai.types import ChatCompletionResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChat:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_completion(self, client: Relaxai) -> None:
        chat = client.chat.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_completion_with_all_params(self, client: Relaxai) -> None:
        chat = client.chat.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                    "annotations": [
                        {
                            "type": "url_citation",
                            "url_citation": {
                                "end_index": 0,
                                "start_index": 0,
                                "title": "title",
                                "url": "url",
                            },
                        }
                    ],
                    "function_call": {
                        "arguments": "arguments",
                        "name": "name",
                    },
                    "multi_content": [
                        {
                            "image_url": {
                                "detail": "detail",
                                "url": "url",
                            },
                            "text": "text",
                            "type": "type",
                        }
                    ],
                    "name": "name",
                    "reasoning_content": "reasoning_content",
                    "refusal": "refusal",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "type",
                            "id": "id",
                            "index": 0,
                        }
                    ],
                }
            ],
            model="model",
            query_stream=True,
            chat_template_kwargs={},
            frequency_penalty=0,
            function_call={},
            functions=[
                {
                    "name": "name",
                    "parameters": {},
                    "description": "description",
                    "strict": True,
                }
            ],
            logit_bias={"foo": 0},
            logprobs=True,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            n=0,
            parallel_tool_calls={},
            prediction={
                "content": "content",
                "type": "type",
            },
            presence_penalty=0,
            reasoning_effort="reasoning_effort",
            response_format={
                "json_schema": {
                    "name": "name",
                    "strict": True,
                    "description": "description",
                },
                "type": "type",
            },
            seed=0,
            stop=["string"],
            store=True,
            body_stream=True,
            stream_options={"include_usage": True},
            temperature=0,
            tool_choice={},
            tools=[
                {
                    "type": "type",
                    "function": {
                        "name": "name",
                        "parameters": {},
                        "description": "description",
                        "strict": True,
                    },
                }
            ],
            top_logprobs=0,
            top_p=0,
            user="user",
            web_search_options={
                "search_context_size": 0,
                "user_location": {
                    "approximate": {
                        "city": "city",
                        "country": "country",
                        "latitude": 0,
                        "longitude": 0,
                        "state": "state",
                    },
                    "type": "type",
                },
            },
        )
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_completion(self, client: Relaxai) -> None:
        response = client.chat.with_raw_response.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_completion(self, client: Relaxai) -> None:
        with client.chat.with_streaming_response.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatCompletionResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChat:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_completion(self, async_client: AsyncRelaxai) -> None:
        chat = await async_client.chat.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_completion_with_all_params(self, async_client: AsyncRelaxai) -> None:
        chat = await async_client.chat.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                    "annotations": [
                        {
                            "type": "url_citation",
                            "url_citation": {
                                "end_index": 0,
                                "start_index": 0,
                                "title": "title",
                                "url": "url",
                            },
                        }
                    ],
                    "function_call": {
                        "arguments": "arguments",
                        "name": "name",
                    },
                    "multi_content": [
                        {
                            "image_url": {
                                "detail": "detail",
                                "url": "url",
                            },
                            "text": "text",
                            "type": "type",
                        }
                    ],
                    "name": "name",
                    "reasoning_content": "reasoning_content",
                    "refusal": "refusal",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "type",
                            "id": "id",
                            "index": 0,
                        }
                    ],
                }
            ],
            model="model",
            query_stream=True,
            chat_template_kwargs={},
            frequency_penalty=0,
            function_call={},
            functions=[
                {
                    "name": "name",
                    "parameters": {},
                    "description": "description",
                    "strict": True,
                }
            ],
            logit_bias={"foo": 0},
            logprobs=True,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            n=0,
            parallel_tool_calls={},
            prediction={
                "content": "content",
                "type": "type",
            },
            presence_penalty=0,
            reasoning_effort="reasoning_effort",
            response_format={
                "json_schema": {
                    "name": "name",
                    "strict": True,
                    "description": "description",
                },
                "type": "type",
            },
            seed=0,
            stop=["string"],
            store=True,
            body_stream=True,
            stream_options={"include_usage": True},
            temperature=0,
            tool_choice={},
            tools=[
                {
                    "type": "type",
                    "function": {
                        "name": "name",
                        "parameters": {},
                        "description": "description",
                        "strict": True,
                    },
                }
            ],
            top_logprobs=0,
            top_p=0,
            user="user",
            web_search_options={
                "search_context_size": 0,
                "user_location": {
                    "approximate": {
                        "city": "city",
                        "country": "country",
                        "latitude": 0,
                        "longitude": 0,
                        "state": "state",
                    },
                    "type": "type",
                },
            },
        )
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_completion(self, async_client: AsyncRelaxai) -> None:
        response = await async_client.chat.with_raw_response.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatCompletionResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_completion(self, async_client: AsyncRelaxai) -> None:
        async with async_client.chat.with_streaming_response.create_completion(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatCompletionResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import deep_research_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import is_given, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._constants import DEFAULT_TIMEOUT
from .._base_client import make_request_options
from ..types.stream_options_param import StreamOptionsParam
from ..types.deepresearch_response import DeepresearchResponse
from ..types.chat_completion_message_param import ChatCompletionMessageParam

__all__ = ["DeepResearchResource", "AsyncDeepResearchResource"]


class DeepResearchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeepResearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/relax-ai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return DeepResearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeepResearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/relax-ai/python-sdk#with_streaming_response
        """
        return DeepResearchResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        messages: Iterable[ChatCompletionMessageParam],
        model: str,
        max_tokens: int | Omit = omit,
        response_format: deep_research_create_params.ResponseFormat | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: bool | Omit = omit,
        stream_options: StreamOptionsParam | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: object | Omit = omit,
        tools: Iterable[deep_research_create_params.Tool] | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeepresearchResponse:
        """
        Performs deep research on a given topic and returns a detailed report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 600
        return self._post(
            "/v1/deep-research",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "max_tokens": max_tokens,
                    "response_format": response_format,
                    "stop": stop,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_p": top_p,
                },
                deep_research_create_params.DeepResearchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeepresearchResponse,
        )


class AsyncDeepResearchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeepResearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/relax-ai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDeepResearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeepResearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/relax-ai/python-sdk#with_streaming_response
        """
        return AsyncDeepResearchResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        messages: Iterable[ChatCompletionMessageParam],
        model: str,
        max_tokens: int | Omit = omit,
        response_format: deep_research_create_params.ResponseFormat | Omit = omit,
        stop: SequenceNotStr[str] | Omit = omit,
        stream: bool | Omit = omit,
        stream_options: StreamOptionsParam | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: object | Omit = omit,
        tools: Iterable[deep_research_create_params.Tool] | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeepresearchResponse:
        """
        Performs deep research on a given topic and returns a detailed report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 600
        return await self._post(
            "/v1/deep-research",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "max_tokens": max_tokens,
                    "response_format": response_format,
                    "stop": stop,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_p": top_p,
                },
                deep_research_create_params.DeepResearchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeepresearchResponse,
        )


class DeepResearchResourceWithRawResponse:
    def __init__(self, deep_research: DeepResearchResource) -> None:
        self._deep_research = deep_research

        self.create = to_raw_response_wrapper(
            deep_research.create,
        )


class AsyncDeepResearchResourceWithRawResponse:
    def __init__(self, deep_research: AsyncDeepResearchResource) -> None:
        self._deep_research = deep_research

        self.create = async_to_raw_response_wrapper(
            deep_research.create,
        )


class DeepResearchResourceWithStreamingResponse:
    def __init__(self, deep_research: DeepResearchResource) -> None:
        self._deep_research = deep_research

        self.create = to_streamed_response_wrapper(
            deep_research.create,
        )


class AsyncDeepResearchResourceWithStreamingResponse:
    def __init__(self, deep_research: AsyncDeepResearchResource) -> None:
        self._deep_research = deep_research

        self.create = async_to_streamed_response_wrapper(
            deep_research.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .stream_options_param import StreamOptionsParam
from .function_definition_param import FunctionDefinitionParam
from .chat_completion_message_param import ChatCompletionMessageParam

__all__ = [
    "ChatCreateCompletionParams",
    "ChatTemplateKwargs",
    "FunctionCall",
    "ParallelToolCalls",
    "Prediction",
    "ResponseFormat",
    "ResponseFormatJsonSchema",
    "ToolChoice",
    "Tool",
    "WebSearchOptions",
    "WebSearchOptionsUserLocation",
    "WebSearchOptionsUserLocationApproximate",
]


class ChatCreateCompletionParams(TypedDict, total=False):
    messages: Required[Iterable[ChatCompletionMessageParam]]

    model: Required[str]

    query_stream: Annotated[bool, PropertyInfo(alias="stream")]
    """If true, server responds as an SSE stream.

    Generators may produce an ergonomic streaming method when this is set.
    """

    chat_template_kwargs: ChatTemplateKwargs

    frequency_penalty: float

    function_call: FunctionCall

    functions: Iterable[FunctionDefinitionParam]

    logit_bias: Dict[str, int]

    logprobs: bool

    max_completion_tokens: int

    max_tokens: int

    metadata: Dict[str, str]

    n: int

    parallel_tool_calls: ParallelToolCalls

    prediction: Prediction

    presence_penalty: float

    reasoning_effort: str

    response_format: ResponseFormat

    seed: int

    stop: SequenceNotStr[str]

    store: bool

    body_stream: Annotated[bool, PropertyInfo(alias="stream")]

    stream_options: StreamOptionsParam

    temperature: float

    tool_choice: ToolChoice

    tools: Iterable[Tool]

    top_logprobs: int

    top_p: float

    user: str

    web_search_options: WebSearchOptions


class ChatTemplateKwargs(TypedDict, total=False):
    pass


class FunctionCall(TypedDict, total=False):
    pass


class ParallelToolCalls(TypedDict, total=False):
    pass


class Prediction(TypedDict, total=False):
    content: Required[str]

    type: Required[str]


class ResponseFormatJsonSchema(TypedDict, total=False):
    name: Required[str]

    strict: Required[bool]

    description: str


class ResponseFormat(TypedDict, total=False):
    json_schema: ResponseFormatJsonSchema

    type: str


class ToolChoice(TypedDict, total=False):
    pass


class Tool(TypedDict, total=False):
    type: Required[str]

    function: FunctionDefinitionParam


class WebSearchOptionsUserLocationApproximate(TypedDict, total=False):
    city: str

    country: str

    latitude: float

    longitude: float

    state: str


class WebSearchOptionsUserLocation(TypedDict, total=False):
    approximate: WebSearchOptionsUserLocationApproximate

    type: str


class WebSearchOptions(TypedDict, total=False):
    search_context_size: int

    user_location: WebSearchOptionsUserLocation



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ToolResponse", "Plot"]


class Plot(BaseModel):
    content_base64: str

    filename: str

    format: str


class ToolResponse(BaseModel):
    execution_time: float

    exit_code: int

    security_checked: bool

    stderr: str

    stdout: str

    success: bool

    plots: Optional[List[Plot]] = None

    violations: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ContentFilterResults", "Hate", "Jailbreak", "Profanity", "SelfHarm", "Sexual", "Violence"]


class Hate(BaseModel):
    filtered: bool

    severity: Optional[str] = None


class Jailbreak(BaseModel):
    detected: bool

    filtered: bool


class Profanity(BaseModel):
    detected: bool

    filtered: bool


class SelfHarm(BaseModel):
    filtered: bool

    severity: Optional[str] = None


class Sexual(BaseModel):
    filtered: bool

    severity: Optional[str] = None


class Violence(BaseModel):
    filtered: bool

    severity: Optional[str] = None


class ContentFilterResults(BaseModel):
    hate: Optional[Hate] = None

    jailbreak: Optional[Jailbreak] = None

    profanity: Optional[Profanity] = None

    self_harm: Optional[SelfHarm] = None

    sexual: Optional[Sexual] = None

    violence: Optional[Violence] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ToolExecuteCodeParams"]


class ToolExecuteCodeParams(TypedDict, total=False):
    code: Required[str]

    lang: Required[str]

    libraries: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .function_call import FunctionCall
from .shared.openai_usage import OpenAIUsage

__all__ = [
    "DeepresearchResponse",
    "Choice",
    "ChoiceMessage",
    "ChoiceMessageAnnotation",
    "ChoiceMessageAnnotationURLCitation",
    "ChoiceMessageToolCall",
    "ChoiceLogprobs",
    "ChoiceLogprobsContent",
    "ChoiceLogprobsContentTopLogprob",
]


class ChoiceMessageAnnotationURLCitation(BaseModel):
    end_index: int

    start_index: int

    title: str

    url: str


class ChoiceMessageAnnotation(BaseModel):
    type: str

    url_citation: Optional[ChoiceMessageAnnotationURLCitation] = None


class ChoiceMessageToolCall(BaseModel):
    function: FunctionCall

    type: str

    id: Optional[str] = None

    index: Optional[int] = None


class ChoiceMessage(BaseModel):
    role: str

    annotations: Optional[List[ChoiceMessageAnnotation]] = None

    content: Optional[str] = None

    function_call: Optional[FunctionCall] = None

    name: Optional[str] = None

    reasoning_content: Optional[str] = None

    refusal: Optional[str] = None

    tool_call_id: Optional[str] = None

    tool_calls: Optional[List[ChoiceMessageToolCall]] = None


class ChoiceLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[str] = None


class ChoiceLogprobsContent(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[ChoiceLogprobsContentTopLogprob]

    bytes: Optional[str] = None


class ChoiceLogprobs(BaseModel):
    content: List[ChoiceLogprobsContent]


class Choice(BaseModel):
    finish_reason: str

    index: int

    message: ChoiceMessage

    logprobs: Optional[ChoiceLogprobs] = None


class DeepresearchResponse(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: str

    system_fingerprint: str

    usage: OpenAIUsage

    service_tier: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Model", "Permission"]


class Permission(BaseModel):
    id: str

    allow_create_engine: bool

    allow_fine_tuning: bool

    allow_logprobs: bool

    allow_sampling: bool

    allow_search_indices: bool

    allow_view: bool

    created: int

    group: object

    is_blocking: bool

    object: str

    organization: str


class Model(BaseModel):
    id: str

    created: int

    http_header: Dict[str, List[str]] = FieldInfo(alias="httpHeader")

    object: str

    owned_by: str

    parent: str

    permission: List[Permission]

    root: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .function_call_param import FunctionCallParam

__all__ = [
    "ChatCompletionMessageParam",
    "Annotation",
    "AnnotationURLCitation",
    "MultiContent",
    "MultiContentImageURL",
    "ToolCall",
]


class AnnotationURLCitation(TypedDict, total=False):
    end_index: Required[int]

    start_index: Required[int]

    title: Required[str]

    url: Required[str]


class Annotation(TypedDict, total=False):
    type: Literal["url_citation"]

    url_citation: AnnotationURLCitation


class MultiContentImageURL(TypedDict, total=False):
    detail: str

    url: str


class MultiContent(TypedDict, total=False):
    image_url: MultiContentImageURL

    text: str

    type: str


class ToolCall(TypedDict, total=False):
    function: Required[FunctionCallParam]

    type: Required[str]

    id: str

    index: int


class ChatCompletionMessageParam(TypedDict, total=False):
    content: Required[str]

    role: Required[str]

    annotations: Iterable[Annotation]

    function_call: FunctionCallParam

    multi_content: Annotated[Iterable[MultiContent], PropertyInfo(alias="MultiContent")]

    name: str

    reasoning_content: str

    refusal: str

    tool_call_id: str

    tool_calls: Iterable[ToolCall]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.openai_usage import OpenAIUsage

__all__ = ["EmbeddingResponse", "Data"]


class Data(BaseModel):
    embedding: List[float]

    index: int

    object: str


class EmbeddingResponse(BaseModel):
    data: List[Data]

    http_header: Dict[str, List[str]] = FieldInfo(alias="httpHeader")

    model: str

    object: str

    usage: OpenAIUsage



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .openai_prompt_tokens_details import OpenAIPromptTokensDetails
from .openai_completion_tokens_details import OpenAICompletionTokensDetails

__all__ = ["OpenAIUsage"]


class OpenAIUsage(BaseModel):
    completion_tokens: int

    completion_tokens_details: OpenAICompletionTokensDetails

    prompt_tokens: int

    prompt_tokens_details: OpenAIPromptTokensDetails

    total_tokens: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .function_call import FunctionCall

__all__ = [
    "ChatCompletionMessage",
    "Annotation",
    "AnnotationURLCitation",
    "MultiContent",
    "MultiContentImageURL",
    "ToolCall",
]


class AnnotationURLCitation(BaseModel):
    end_index: int

    start_index: int

    title: str

    url: str


class Annotation(BaseModel):
    type: Optional[Literal["url_citation"]] = None

    url_citation: Optional[AnnotationURLCitation] = None


class MultiContentImageURL(BaseModel):
    detail: Optional[str] = None

    url: Optional[str] = None


class MultiContent(BaseModel):
    image_url: Optional[MultiContentImageURL] = None

    text: Optional[str] = None

    type: Optional[str] = None


class ToolCall(BaseModel):
    function: FunctionCall

    type: str

    id: Optional[str] = None

    index: Optional[int] = None


class ChatCompletionMessage(BaseModel):
    content: str

    role: str

    annotations: Optional[List[Annotation]] = None

    function_call: Optional[FunctionCall] = None

    multi_content: Optional[List[MultiContent]] = FieldInfo(alias="MultiContent", default=None)

    name: Optional[str] = None

    reasoning_content: Optional[str] = None

    refusal: Optional[str] = None

    tool_call_id: Optional[str] = None

    tool_calls: Optional[List[ToolCall]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from relaxai import Relaxai, AsyncRelaxai
from tests.utils import assert_matches_type
from relaxai.types import DeepresearchResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDeepResearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Relaxai) -> None:
        deep_research = client.deep_research.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Relaxai) -> None:
        deep_research = client.deep_research.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                    "annotations": [
                        {
                            "type": "url_citation",
                            "url_citation": {
                                "end_index": 0,
                                "start_index": 0,
                                "title": "title",
                                "url": "url",
                            },
                        }
                    ],
                    "function_call": {
                        "arguments": "arguments",
                        "name": "name",
                    },
                    "multi_content": [
                        {
                            "image_url": {
                                "detail": "detail",
                                "url": "url",
                            },
                            "text": "text",
                            "type": "type",
                        }
                    ],
                    "name": "name",
                    "reasoning_content": "reasoning_content",
                    "refusal": "refusal",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "type",
                            "id": "id",
                            "index": 0,
                        }
                    ],
                }
            ],
            model="model",
            max_tokens=0,
            response_format={
                "json_schema": {
                    "description": "description",
                    "name": "name",
                    "schema": "schema",
                    "strict": True,
                },
                "type": "type",
            },
            stop=["string"],
            stream=True,
            stream_options={"include_usage": True},
            temperature=0,
            tool_choice={},
            tools=[
                {
                    "type": "type",
                    "function": {
                        "name": "name",
                        "parameters": {},
                        "description": "description",
                        "strict": True,
                    },
                }
            ],
            top_p=0,
        )
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Relaxai) -> None:
        response = client.deep_research.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deep_research = response.parse()
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Relaxai) -> None:
        with client.deep_research.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deep_research = response.parse()
            assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDeepResearch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncRelaxai) -> None:
        deep_research = await async_client.deep_research.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRelaxai) -> None:
        deep_research = await async_client.deep_research.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                    "annotations": [
                        {
                            "type": "url_citation",
                            "url_citation": {
                                "end_index": 0,
                                "start_index": 0,
                                "title": "title",
                                "url": "url",
                            },
                        }
                    ],
                    "function_call": {
                        "arguments": "arguments",
                        "name": "name",
                    },
                    "multi_content": [
                        {
                            "image_url": {
                                "detail": "detail",
                                "url": "url",
                            },
                            "text": "text",
                            "type": "type",
                        }
                    ],
                    "name": "name",
                    "reasoning_content": "reasoning_content",
                    "refusal": "refusal",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "type",
                            "id": "id",
                            "index": 0,
                        }
                    ],
                }
            ],
            model="model",
            max_tokens=0,
            response_format={
                "json_schema": {
                    "description": "description",
                    "name": "name",
                    "schema": "schema",
                    "strict": True,
                },
                "type": "type",
            },
            stop=["string"],
            stream=True,
            stream_options={"include_usage": True},
            temperature=0,
            tool_choice={},
            tools=[
                {
                    "type": "type",
                    "function": {
                        "name": "name",
                        "parameters": {},
                        "description": "description",
                        "strict": True,
                    },
                }
            ],
            top_p=0,
        )
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRelaxai) -> None:
        response = await async_client.deep_research.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deep_research = await response.parse()
        assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRelaxai) -> None:
        async with async_client.deep_research.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deep_research = await response.parse()
            assert_matches_type(DeepresearchResponse, deep_research, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["FunctionCall"]


class FunctionCall(BaseModel):
    arguments: Optional[str] = None

    name: Optional[str] = None
